"""
Template configurations for additional questions etc.

All questions should have sensible defaults.
Questions without configured defaults will default to empty ('').

You can access data from plugins etc. by using
data[key]. Keys that do not exist will default to "".
"""
from pathlib import Path

def config(template_path: Path, project_path: Path, data: dict):
    questions = {
        "author": "Author (default: ''): ",
        "author_email": "Author Email (default: ''): ",
        "license": "License (default: 'MIT'): ",
        "description": "Short description (default: ''): ",
        "keywords": "Keywords? (default: ''): ",
        "project_url": f"Project URL? (default: '{data['git_remote']}'): ",
        "project_source": f"Project Source? (default: '{data['git_remote']}'): ",
    }

    defaults = {
        "license": "MIT",
        "description": "",
        "keywords": "",
        "project_url": data["git_remote"],
        "project_source": data["git_remote"]
    }

    return questions, defaults, data



import sys
import importlib
import mimetypes
from collections import defaultdict
from pathlib import Path
import jinja2

import skelgen.plugins

def run_config(template_path: Path, project_path: Path, data: dict):
    """
    Ask additional questions in a template.
    Template-specific questions are contained in
    ${template_path}/___questions.py (3 underscores).
    """
    print(data)

    sys.path.append(str(template_path))
    try:
        template = importlib.import_module("___template")
    except ModuleNotFoundError:
        # No additional questions
        return data

    questions, defaults, data = template.config(template_path, project_path, data)

    for var, question in questions.items():
        data[var] = input(question)
        if data[var] == "":
            data[var] = defaults.get(var, "")

    return data

def fill_template(template_path: Path, project_path: Path, **kwargs):
    data = {
        "template_path": template_path,
        "project_path": project_path,
        **kwargs,
        **skelgen.plugins.run_plugins(template_path, project_path)
    }
    data = run_config(template_path, project_path, defaultdict(str, data))

    print("Scaffolding...")

    for path in template_path.glob("**/*"):
        rel_path = path.relative_to(template_path)
        path_templated = jinja2.Template(str(rel_path)).render(data)
        if path_templated.endswith(".___tmpl"):
            path_templated = path_templated.split(".___tmpl")[0]

        if Path(path_templated).name.startswith("___"):
            # Skip files prefixed with ___ (e.g. template config)
            continue

        if path.is_dir():
            Path.mkdir(project_path / path_templated, parents=True, exist_ok=True)
            print("Copied", project_path / path_templated)
        else:
            try:
                with open(path, "r") as content:
                    template = jinja2.Template(content.read())

                if data["overwrite"] or not (project_path / path_templated).exists():
                    with open(str(project_path / path_templated), "w") as out:
                        out.write(template.render(data))
                    print("Copied", project_path / path_templated)
                else:
                    print(f"File {project_path / path_templated} exists, skipping")
            except UnicodeDecodeError:
                # File is not text, should not be templated
                pass

    print("---")
    print("Done!")



from pathlib import Path
from skelgen.plugins.git import GitPlugin

def run_plugins(template_path: Path, project_path: Path):
    data = {}
    data.update(GitPlugin().get_data(project_path))
    return data



import sys
import argparse
from pathlib import Path

import skelgen.template
from skelgen.exception import TemplateNotExistsError

def main():
    templates = Path(__file__).parent / "templates"

    parser = argparse.ArgumentParser()
    parser.add_argument("template", help="The template to scaffold with.")
    parser.add_argument("--name", help="The name of the project.")
    parser.add_argument("--overwrite", type=bool, help="Overwrite existing files.")
    parser.add_argument("--path",
                        default=Path.cwd(),
                        help="The path to the project.")

    args = parser.parse_args()

    if args.name is None:
        project_name = input("Name: ")
    else:
        project_name = args.name

    if not project_name:
        print("Project name cannot be empty.")
        sys.exit(1)

    template_folder = templates / args.template
    if not template_folder.exists():
        sys.tracebacklimit = 0
        raise TemplateNotExistsError("The specified template does not exist.")

    project_folder = Path(args.path)
    Path.mkdir(project_folder, parents=True, exist_ok=True)

    skelgen.template.fill_template(template_folder,
                                project_folder,
                                project_name=project_name,
                                overwrite=args.overwrite)



from pathlib import Path
from collections import defaultdict
import git

class GitPlugin:
    def get_data(self, template_path: Path) -> dict:
        data = defaultdict(str)

        try:
            repo = git.Repo(template_path)
        except git.exc.InvalidGitRepositoryError:
            data["is_git_repo"] = False
            return data

        data["git_is_repo"] = True
        remote = repo.remote()
        if remote is not None:
            if len(list(remote.urls)) > 0:
                data["git_remote"] = list(remote.urls)[0]
        else:
            pass

        return data



import setuptools
from skelgen.version import VERSION
from distutils.command.install import INSTALL_SCHEMES

for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

setuptools.setup(
    version=VERSION
)
